@@ -3,53 +3,91 @@
 import re
 
 from lookuper import (
-    ALWAYS,
+    ANY,
     GLOBSTAR,
-    NEVER,
     STAR,
     lookup,
     lookup_data,
     lookup_target,
-    match,
-    match_key,
-    match_value,
+    Match,
 )
 
 import pytest
 
 
 @pytest.mark.parametrize(
-    'lookup_type, string', [(STAR, '*'), (GLOBSTAR, '**')],
+    'constant, args, expected',
+    [(ANY, (), True), (ANY, (0,), True), (ANY, (0, 0,), True)],
 )
-def test_lookup_types(lookup_type, string):
-    """Lookup types should behave like strings."""
-    assert lookup_type == string
+def test_match_constants(constant, args, expected):
+    """Calling match constants should always return the same value."""
+    result = constant(*args)
+
+    assert result == expected
 
 
 @pytest.mark.parametrize(
-    'func, args, matches',
-    [(ALWAYS, (), True), (ALWAYS, (0,), True), (ALWAYS, (0, 0,), True)],
+    'func, key, value, expected',
+    [
+        (Match(), 'a', 1, True),
+        (Match(key='a'), 'a', 1, True),
+        (Match(key='b'), 'a', 1, False),
+        (Match(value=1), 'a', 1, True),
+        (Match(value=2), 'a', 1, False),
+        (Match('a', 1), 'a', 1, True),
+        (Match('b', 1), 'a', 1, False),
+        (Match('a', 2), 'a', 1, False),
+    ],
 )
-def test_match_functions(func, args, matches):
-    """Calling match functions should return the expected result."""
-    result = func(*args)
+def test_match_call(func, key, value, expected):
+    """Matching key/value pairs should return the expected result."""
+    result = func(key, value)
 
-    assert result == matches
+    assert result == expected
 
 
 @pytest.mark.parametrize(
-    'targets, data, matches',
+    'func, expected',
+    [
+        (ANY, 'ANY'),
+        (Match(), 'Match(key=ANY, value=ANY)'),
+        (Match(1), 'Match(key=1, value=ANY)'),
+        (Match('a'), "Match(key='a', value=ANY)"),
+        (Match(value=1), 'Match(key=ANY, value=1)'),
+        (Match(value='a'), "Match(key=ANY, value='a')"),
+        (Match(1, 2), 'Match(key=1, value=2)'),
+    ],
+)
+def test_match_repr(func, expected):
+    """Representating match functions should return the expected result."""
+    result = repr(func)
+
+    assert result == expected
+
+
+@pytest.mark.parametrize(
+    'constant, string', [(STAR, '*'), (GLOBSTAR, '**')],
+)
+def test_lookup_constants(constant, string):
+    """Lookup constants should behave like strings."""
+    assert constant == string
+
+
+@pytest.mark.parametrize(
+    'targets, data, expected',
     [
         ([], None, [None]),
         ([], {'a': 1}, [{'a': 1}]),
         (['a'], {'a'}, ['a']),
+        (['0'], {'0'}, ['0']),
+        ([0], {0}, [0]),
         (['a'], {'a': 1}, [1]),
         (['b'], {'a': 1}, []),
         (['a'], {'a': {'b': 1}}, [{'b': 1}]),
         (['a', 'b'], {'a': {'b': 1}}, [1]),
         (['a', 'b'], {'a': 1}, []),
         ([0], ['a'], ['a']),
-        (['0'], ['a'], ['a']),
+        (['0'], ['a'], []),
         ([1], ['a'], []),
         ([0], [['a']], [['a']]),
         ([0, 0], [['a']], ['a']),
@@ -89,26 +127,19 @@ def test_match_functions(func, args, matches):
         (['a', '**', 'd'], {'a': {'b': [{'d': 1}]}}, [1]),
         (['a', '**', 'd'], {'a': [{'c': {'d': 1}}]}, [1]),
         (['a', '**', 'd'], {'a': [[{'d': 1}]]}, [1]),
-        (['a', match_key(0), 'b'], {'a': [{'b': 2}]}, [2]),
-        (['a', match_value(1)], {'a': {'b': 1}}, [1]),
-        (['a.b'], {'a': {'b': 1}}, [1]),
-        (['a.b'], {'a': 1}, []),
-        ([r'\*'], {'*': 1}, [1]),
-        ([r'a\.b'], {'a.b': 1}, [1]),
-        ([r'a\.b\.c'], {'a.b.c': 1}, [1]),
-        ([r'a\.b.c'], {'a.b': {'c': 1}}, [1]),
-        ([r'a.b\.c'], {'a': {'b.c': 1}}, [1]),
+        (['a', Match(0), 'b'], {'a': [{'b': 2}]}, [2]),
+        (['a', Match(value=1)], {'a': {'b': 1}}, [1]),
     ],
 )
-def test_lookup(targets, data, matches):
+def test_lookup(targets, data, expected):
     """Looking up a target from data should yield the expected results."""
     result = list(lookup(data, *targets))
 
-    assert result == matches
+    assert result == expected
 
 
 @pytest.mark.parametrize(
-    'data, matches',
+    'data, expected',
     [
         # Default
         (None, []),
@@ -127,15 +158,15 @@ def test_lookup(targets, data, matches):
         ({'a'}, [('a', 'a')]),
     ],
 )
-def test_lookup_data(data, matches):
+def test_lookup_data(data, expected):
     """Looking up a data should yield the expected results."""
     result = list(lookup_data(data))
 
-    assert result == matches
+    assert result == expected
 
 
 @pytest.mark.parametrize(
-    'target, key, value, matches',
+    'target, key, value, expected',
     [
         # Default
         ('a', None, {'a': 1}, [('a', 1)]),
@@ -145,73 +176,35 @@ def test_lookup_data(data, matches):
         (['a', 'b'], None, {'a': {'b': 1}}, [('b', 1)]),
         # String
         ('a', None, {'a': 1}, [('a', 1)]),
-        ('a.b', None, {'a': {'b': 1}}, [('b', 1)]),
+        ('b', None, {'a': 1}, []),
+        ('*', None, {'a': 1}, [('a', 1)]),
+        (r'\*', None, {'a': 1}, []),
+        (r'\*', None, {'*': 1}, [('*', 1)]),
+        ('**', 'a', {'b': 1}, [('a', {'b': 1}), ('b', 1)]),
+        (r'\**', 'a', {'b': 1}, []),
+        (r'\**', None, {'**': 1}, [('**', 1)]),
+        (r'*\*', None, {'**': 1}, [('**', 1)]),
+        (r'\*\*', None, {'**': 1}, [('**', 1)]),
         # Callable
-        (match_key('a'), None, {'a': 1}, [('a', 1)]),
-        (match_key('b'), None, {'a': 1}, []),
-        (match_value(1), None, {'a': 1}, [('a', 1)]),
-        (match_value(2), None, {'a': 1}, []),
+        (str.isalpha, None, {'a': 1}, [('a', 1)]),
+        (str.isdigit, None, {'a': 1}, []),
+        # Match
+        (Match('a'), None, {'a': 1}, [('a', 1)]),
+        (Match('b'), None, {'a': 1}, []),
+        (Match(value=1), None, {'a': 1}, [('a', 1)]),
+        (Match(value=2), None, {'a': 1}, []),
         # RePattern
         (re.compile(r'\w'), None, {'a': 1}, [('a', 1)]),
         (re.compile(r'\d'), None, {'a': 1}, []),
         # STAR
         (STAR, None, {'a': 1}, [('a', 1)]),
+        (STAR, None, {'a': 1, 'b': 2}, [('a', 1), ('b', 2)]),
         # GLOBSTAR
         (GLOBSTAR, 'a', {'b': 1}, [('a', {'b': 1}), ('b', 1)]),
     ],
 )
-def test_lookup_target(target, key, value, matches):
+def test_lookup_target(target, key, value, expected):
     """Looking up a target from data should yield the expected results."""
     result = list(lookup_target(target, key, value))
 
-    assert result == matches
-
-
-@pytest.mark.parametrize(
-    'func, key, value, matches',
-    [
-        (match(), 'a', 1, True),
-        (match(key='a'), 'a', 1, True),
-        (match(value=1), 'a', 1, True),
-        (match('a', 1), 'a', 1, True),
-        (match('b', 1), 'a', 1, False),
-        (match('a', 2), 'a', 1, False),
-    ],
-)
-def test_match(func, key, value, matches):
-    """Matching key/value pairs should return the expected data."""
-    result = func(key, value)
-
-    assert result == matches
-
-
-@pytest.mark.parametrize(
-    'target, key, value, matches',
-    [
-        (ALWAYS, 'a', 1, True),
-        (NEVER, 'a', 1, False),
-        ('a', 'a', 1, True),
-        ('b', 'a', 1, False),
-    ],
-)
-def test_match_key(target, key, value, matches):
-    """Matching a key should return the expected value."""
-    result = match_key(target)(key, value)
-
-    assert result == matches
-
-
-@pytest.mark.parametrize(
-    'target, key, value, matches',
-    [
-        (ALWAYS, 0, 'a', True),
-        (NEVER, 0, 'a', False),
-        ('a', 0, 'a', True),
-        ('b', 0, 'a', False),
-    ],
-)
-def test_match_value(target, key, value, matches):
-    """Matching a value should return the expected key."""
-    result = match_value(target)(key, value)
-
-    assert result == matches
+    assert result == expected
